/*****************************************************************************
 * ADT Example Program:		ADT_L1_1553_ex_BIT_BusAB_Wrap.c
 *
 * Copyright (c) 2010, Alta Data Technologies LLC (ADT), All Rights Reserved.
 * Use of this software is subject to the ADT Software License (latest
 * revision), US Government or local laws and the ADT Terms and Conditions 
 * of Sale (latest revision).
 *
 * Description:
 *		This program demonstrates how to perform a BUS A to BUS B wrap test.
 *
 *****************************************************************************/
#include <stdio.h>
#include <memory.h>
#include "ADT_L1.h"

/* The DEVICE ID is a 32-bit value that identifies the following:
 *		bits 28-31 = Backplane Type (0 = Simulated, 1 = PCI)
 *		bits 20-27 = Board Type (0 = SIM-1553, 1 = TEST-1553, 2 = PMC-1553, 3 = PC104P-1553, 4 = PCI-1553)
 *		bits 16-19 = Board Number (0 to 15)
 *      bits 8-15 = Channel Type (0x10 = 1553)
 *		bits 0-7 = Channel Number (0 to 255)
 *
 * A device ID of 0x10201000 specifies the first 1553 channel of the
 * first ADT PMC-1553 board found.
 * A device ID of 0x10201001 specifies the second 1553 channel of the
 * first ADT PMC-1553 board found.
 */
#define DEVID (ADT_PRODUCT_PCCD1553 | ADT_DEVID_BOARDNUM_01 | ADT_DEVID_CHANNELTYPE_1553 |ADT_DEVID_CHANNELNUM_01)


ADT_L0_UINT32 ADT_BIT_1553_BusAB_Wrap(ADT_L0_UINT32 dev_id);

#define NUM_TESTS 1000

int main() {
	ADT_L0_UINT32 status, i;
	ADT_L0_UINT32 l0Version, l1Version;
	ADT_L0_UINT16 peVersion;
	char c;

	printf("********** ADT L1 API Example Program **********\n\n");
	printf("Initializing . . . ");
	status = ADT_L1_1553_InitDefault(DEVID, 10);
	if (status == ADT_SUCCESS) {
		printf("Success.\n");

		printf("Checking Protocol Engine (PE) and API versions . . . ");
		status = ADT_L1_GetVersionInfo(DEVID, &peVersion, &l0Version, &l1Version);
		if (status == ADT_SUCCESS) {
			printf("Success.\n");
			printf("   PE version = %04X\n", peVersion);
			printf("   L0 API version = %d.%d.%d.%d\n", (l0Version & 0xFF000000) >> 24, 
				(l0Version & 0x00FF0000) >> 16, (l0Version & 0x0000FF00) >> 8, l0Version & 0x000000FF);
			printf("   L1 API version = %d.%d.%d.%d\n", (l1Version & 0xFF000000) >> 24, 
				(l1Version & 0x00FF0000) >> 16, (l1Version & 0x0000FF00) >> 8, l1Version & 0x000000FF);
		}
		else printf("FAILURE - Error = %d\n", status);

		printf("\n\nHit a key to run Bus A to Bus B Wrap Test:");
		scanf("%c", &c);

		printf("Running %d Test Iterations . . . ", NUM_TESTS);
		for (i=0; i<NUM_TESTS; i++) {
			status = ADT_BIT_1553_BusAB_Wrap(DEVID);
			if (status != ADT_SUCCESS) break;
		}
		if (status == ADT_SUCCESS) printf("TEST PASSED\n");
		else printf("TEST FAILED on iteration %d\n", i+1);

		/* Close and exit */
		printf("\nClosing . . . ");
		status = ADT_L1_CloseDevice(DEVID);
		if (status == ADT_SUCCESS) printf("Success.\n");
		else printf("FAILURE - Error = %d\n", status);
	}
	else printf("FAILURE - Error = %d\n", status);

	printf("\n***************** Finished *******************\n");
	printf("Hit ENTER to exit.\n");
	scanf("%c", &c);

	return( 0 );
}

/**************************************************************************
 * FUNCTION:  ADT_BIT_1553_BusAB_Wrap
 *
 * This function requires that Bus A be connected to Bus B (through a
 * transformer-coupled bus with proper termination).  NOTE THAT THIS IS
 * AN INVALID BUS CONNECTION - VIOLATES PARAGRAPH 4.6.3 OF THE MIL-STD-1553
 * SPECIFICATION.
 *
 * This function uses the BC and BM, sends a message on Bus A and checks 
 * that it was seen on Bus B, verifies that the command and data words are 
 * correct.
 *
 **************************************************************************/
ADT_L0_UINT32 ADT_BIT_1553_BusAB_Wrap(ADT_L0_UINT32 dev_id)
{
	ADT_L0_UINT32 status, result, i, isRunning, numMsgs, temp, rtAddr;
	ADT_L1_1553_BC_CB myBCCB;
	ADT_L1_1553_CDP myCdp;
	ADT_L1_1553_CDP bmMessages[10];

	result = ADT_SUCCESS;

	/* Setup BM to monitor the test */
	for (rtAddr=0; rtAddr<32; rtAddr++) {
		status = ADT_L1_1553_BM_FilterWrite(dev_id, rtAddr, 0xFFFFFFFF, 0xFFFFFFFF);
		if (status != ADT_SUCCESS) break;
	}
	if (status != ADT_SUCCESS) return(status);

	status = ADT_L1_1553_BM_BufferCreate(dev_id, 10);
	if (status != ADT_SUCCESS) return(status);

	/* Configure decoder to ignore traffic on Bus A, only listen on Bus B. */
	status = ADT_L1_ReadDeviceMem32(dev_id, ADT_L1_1553_PE_ROOT_CSR, &temp, 1);
	if (status != ADT_SUCCESS) return(status);

	temp |= ADT_L1_1553_PECSR_BUS_B_ONLY;

	status = ADT_L1_WriteDeviceMem32(dev_id, ADT_L1_1553_PE_ROOT_CSR, &temp, 1);
	if (status != ADT_SUCCESS) return(status);

	/* Setup BC to send a message */
	/* BC Initialization - max 10 messages, 1 minor per major, BC CSR 0 */
	status = ADT_L1_1553_BC_Init(dev_id, 10, 1, 0);
	if (status != ADT_SUCCESS) return(status);

	/* Allocate BCCB and one CDPs for message 0 */
	status = ADT_L1_1553_BC_CB_CDPAllocate(dev_id, 0, 1);
	if (status != ADT_SUCCESS) return(status);

	/* Define the BCCB for message 0 */
	memset(&myBCCB, 0, sizeof(myBCCB));

	myBCCB.CMD1Info = 0x0820;						/* BCRT 1-R-1-32 on Bus A */
	myBCCB.Csr = ADT_L1_1553_BC_CB_CSR_TYPE_BCRT |
				 ADT_L1_1553_BC_CB_CSR_BUSA;
	myBCCB.DelayTime = 1000;						/* 100.0 us inter-message gap (dead-bus time) 
												     * (irrelevant because we stop after this message) */
	myBCCB.NextMsgNum = ADT_L1_1553_BC_NO_NEXT_MSG;	/* Stop BC after this message */
			
	status = ADT_L1_1553_BC_CB_Write(dev_id, 0, &myBCCB);
	if (status != ADT_SUCCESS) return(status);

	/* Write the data buffer (CDP) for message 0 */
	memset(&myCdp, 0, sizeof(myCdp));
	for (i=0; i<32; i++)
		myCdp.DATAinfo[i] = 0x00001100 + i;  /* This is the data we will expect to see with BM */
	status = ADT_L1_1553_BC_CB_CDPWrite(dev_id, 0, 0, &myCdp);
	if (status != ADT_SUCCESS) return(status);

	/* Start BM */
	status = ADT_L1_1553_BM_Start(dev_id);
	if (status != ADT_SUCCESS) return(status);

	/* Start BC */
	status = ADT_L1_1553_BC_Start(dev_id, 0);
	if (status != ADT_SUCCESS) return(status);

	/* Wait for BC to stop (or timeout) */
	i = 0;
	isRunning = 1;
	while (isRunning) {
		status = ADT_L1_1553_BC_IsRunning(dev_id, &isRunning);
		ADT_L1_msSleep(1);  i++;
		if (i > 10) {
			status = ADT_ERR_TIMEOUT;
			isRunning = 0;
		}
	}
	if (status != ADT_SUCCESS) return(status);

	/* Read the message from the BM */
	numMsgs = 0;
	status = ADT_L1_1553_BM_ReadNewMsgs(dev_id, 10, &numMsgs, bmMessages);
	if (status != ADT_SUCCESS) return(status);

	/* We expect to see ONE message */
	if (numMsgs == 1) {

		/* Should see BCRT message, NO RESP error, two-bus error, bus B, 32 data words */
		if (bmMessages[0].CDPStatusWord == 0x08000520) {

			/* Check for correct 1553 Command Word */
			if ((bmMessages[0].CMD1info & 0x0000FFFF) == (myBCCB.CMD1Info & 0x0000FFFF)) {

				/* Check for correct 1553 Data Words */
				for (i=0; i<32; i++) {
					if ((bmMessages[0].DATAinfo[i] & 0x0000FFFF) != (myCdp.DATAinfo[i] & 0x0000FFFF))
						result = ADT_FAILURE;
				}
			}
			else result = ADT_FAILURE;
		} 
		else result = ADT_FAILURE;
	}
	else result = ADT_FAILURE;

	/* Stop the BM */
	status = ADT_L1_1553_BM_Stop(dev_id);
	if (status != ADT_SUCCESS) return(status);

	/* Free BC memory */
	status = ADT_L1_1553_BC_Close(dev_id);
	if (status != ADT_SUCCESS) return(status);

	/* Free BM memory */
	status = ADT_L1_1553_BM_BufferFree(dev_id);
	if (status != ADT_SUCCESS) return(status);

	return(result);
}